open Base
open Printf
open Syslog
open Okweb

module Daemon = Okauth_server.Daemon.Make(struct
    let db_path = Okauth_server.db_path
    let log msg = syslog msg
  end)

let handle_error ex =
  let exn_uuid = Uuid_unix.create () |> Uuid.to_string in
  sprintf "%s: %s" exn_uuid (Exn.to_string ex) |> syslog;
  let body = sprintf "An exception occurred: %s. Please check logs." exn_uuid in
  Lwt.return Response.(
      of_status 500
      |> with_header ~key:"X-Exception-ID" ~value:exn_uuid
      |> with_string_body body)

let get_host_port () =
  let open Core in
  let from_env (type t) (module T: Stringable with type t = t) ~key ~default =
    let value_opt = Sys.getenv key in
    try
      value_opt |> Option.map ~f:T.of_string |> Option.value ~default
    with
    | Failure _ ->
      sprintf "Invalid %s: %s. Using %s instead" key (Option.value value_opt ~default:"") (T.to_string default) |> syslog;
      default
  in
  let host = from_env (module String) ~key:"HOST" ~default:"localhost" in
  let port = from_env (module Int) ~key:"PORT" ~default:4243 in
  (host, port)

let () =
  openlog ~id:"okauthd" ~options:[ Open_option.PID; Open_option.PERROR; ] ();
  match Daemon.init () with
  | Result.Error err ->
    syslog err;
    Caml.exit 1
  | Result.Ok _ ->
    let (address, port) = get_host_port () in
    sprintf "Starting OkAuth Daemon v%s on %s:%d" [%blob "../../version"] address port |> syslog;
    let handler =
      Okweb.Middleware.Log.middleware ~log:syslog begin fun req ->
        try Daemon.process_request req
        with | ex -> handle_error ex
      end in
    Lwt_main.run (Server.listen ~address ~port handler);
